classdef CurveType_NS < handle
    %CURVETYPE_PD Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        branches
        
    end
    
    methods
        function obj = CurveType_NS()
           obj.branches = struct(); 
            
           obj.branches.NS = { Branch('NS','NS', @init_NSm_NSm , 1 , false,'D') };
           
           obj.branches.CH = { Branch('CH','NS', @init_NSm_NSm , 1 , false,'D') };
           obj.branches.R1 = { Branch('R1','NS', @init_NSm_NSm , 1 , false,'D') };
           obj.branches.R2 = { Branch('R2','NS', @init_NSm_NSm , 1 , false,'D') ,...
                               Branch('R2','NS' ,@init_R2_NS2m , 2 , true ,'eps')};
           obj.branches.R3 = { Branch('R3','NS', @init_NSm_NSm , 1 , false,'D') ,...
                               Branch('R3','NS', @init_R3_NS3m , 3 , false,'eps+bps')};
           obj.branches.R4 = { Branch('R4','NS', @init_NSm_NSm , 1 , false,'D') ,...
                               Branch('R4','NS', @init_R4_NS4m , 4 , true,'eps+bps')};
                           
           obj.branches.LPPD ={Branch('LPPD','NS',@init_LPPD_NS2m,2, true,'eps') };
           
           obj.branches.LPNS = { Branch('LPNS','NS', @init_NSm_NSm , 1 , false,'D') };
           
           obj.branches.PDNS = { Branch('PDNS','NS',  @init_NSm_NSm , 1 , false,'D'),...
                                 Branch('PDNS','NS',@init_PDNS_NS2m , 2, false,'eps') };
                             
           obj.branches.NSNS = { Branch('NSNS','NS', @init_NSm_NSm , 1 , false,'D') ,...
                                 Branch('NSNS','NS', @init_NSm_NSm , 1 , false,'D') }; %FIXME: verwarring hier
           
        end
        
        
        function string = getName(obj)
            string = 'Neimark-Sacker';
        end
        function string = getLabel(obj)
            string = 'NS';
        end
        function col = getDefaultColor(obj)
           col =  'magenta';
        end
        function list = getDependantGlobalVars(obj)
            list = {'cds' , 'nsmds'};
        end
        function [x0,v0] = performInit(obj, session, curve, x0 , par, systemhandle , varargin)
            initpointtag = session.getPointType().getLabel();
            n = session.starterdata.getSetting('iterationN');
            option_eps = 0.0001;
            
            switch initpointtag 
                case 'PDNS'
                    [x0,v0] = init_PDNS_NS2m(systemhandle,option_eps, x0, par ,session.starterdata.getActiveParams(),n);                
                case 'LPPD'
                    [x0,v0] = init_LPPD_NS2m(systemhandle,option_eps, x0, par ,session.starterdata.getActiveParams(),n);
                case 'R2'
                    [x0,v0] = init_R2_NS2m(systemhandle,option_eps,  x0, par ,session.starterdata.getActiveParams(),n);                    
                case 'R3'
                    [x0,v0] = init_R3_NS3m(systemhandle,option_eps, x0, par ,session.starterdata.getActiveParams(),n);  %VARARGIN: branchparams doorgeven
                case 'R4'
                    [x0,v0] = init_R4_NS4m(systemhandle,option_eps, x0, par ,session.starterdata.getActiveParams(),n);  %VARARGIN; branchparams doorgeven                    
                case 'NS'
                    [x0,v0] = init_NSm_NSm(systemhandle, x0, par ,session.starterdata.getActiveParams(),n); 
                otherwise
                    errordlg([mfilename ':  PointType not supported']);
                    error('panic');
            end
            
        end
        function bool = supportPoint(obj, pointtag)
            bool = ismember(pointtag, {'PDNS','LPPD' , 'R2','R3','R4' , 'NS'});
        end          
        function curvedef = getCurveDefinition(obj)
            curvedef = @neimarksackermap;
        end

        
        function list = getNumericVarList(obj)
            list = {'coordinates','parameters','testfunctions','multipliers' ,'current_stepsize' ,  'user_functions' , 'npoints'};
        end            
        %------------------------------------------------------
        function list  = allowedStarterOptions(obj)
            list = {'coordinates' , 'parameters' , 'ADnumber' , 'jacobian' , 'iterationN' , 'multipliers' ,'userfunction' , 'testfunctions' , 'amplitude' };
        end
        
        function [list,mindim] = getTestfunctions(obj)
           list =   {'CH', 'PDNS', 'LPNS', 'R1', 'NSNS', 'R2', 'R3', 'R4'};
           mindim = [ 2   ,  3   ,   3   ,  2  ,   4   ,   2  ,  2  , 2  ];
        end  
        
        
        function nr = getReqNrFreeParams(obj)
           nr = 2; 
        end       
        
        
        function list = getBranches(obj, pointtag)
           if (isfield(obj.branches,pointtag))
               list = obj.branches.(pointtag);
           else
               list = {};
           end
            
        end   
        function mat = getSingMatrix(obj)
           out = feval(obj.getCurveDefinition());
           mat = feval(out{9});
        end
    end
        
end
