classdef NumericVarList < handle
    %NUMERICVARLIST Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        varlist
        handlelist
        startdata
        
    end
    events
        structureChanged
        settingChanged
    end
    
    methods
        
        function obj = NumericVarList(varargin)
            
            if (nargin == 1)
                session = varargin{1};
                system = session.getSystem();
                curvetype = session.getCurveType();
                
                obj.startdata = session.getStartData();
                
                session.addlistener('newSystem' , @(o,e) obj.changed(session));
                session.addlistener('curveChanged' , @(o,e) obj.changed(session));
                obj.startdata.addlistener('settingChanged' ,  @(o,e) obj.changed(session)); %FIXME ! too heavy ?
                
            else
                system = varargin{1};
                curvetype = varargin{2};
            end
            obj.configureVarList(system,curvetype);
        end
        
        function changed(obj,session)
           printc(mfilename, 'changed: reloading'); 
           system = session.getSystem();
           curvetype = session.getCurveType();
           obj.configureVarList(system,curvetype); 
           obj.notify('structureChanged');
        end
        
        function disableAll(obj)
            names = fieldnames(obj.varlist);
            for i = 1:length(names)
               obj.varlist.(names{i}) = 0; 
            end
        end
        
        function clearHandles(obj)
            global sOutput;  sOutput.enabled = 0;
            
            list = fieldnames(obj.handlelist);
            for i = 1:length(list)
               if (isstruct(obj.handlelist.(list{i})))
                    sublist = fieldnames(obj.handlelist.(list{i}));
                    for j = 1:length(sublist)
                        obj.handlelist.(list{i}).(sublist{j}) = [];
                    end
               else
                   obj.handlelist.(list{i}) = [];
               end
            end
        end
        
        function configureVarList(obj,system,curvetype)
                
            varsenabled  = curvetype.getNumericVarList();
            printc(mfilename, 'configureVarList');
            obj.varlist = struct();
            obj.handlelist = struct();
             
             
            for i = 1:length(varsenabled)  %ToFix...  bijhouden per curvetype |||  session ?
                obj.varlist.(varsenabled{i}) = 1;  %TODO: if notset , set
            end
            

            
            coordlist = system.getCoordinateList();
            obj.handlelist.coordinates = struct();
            for i = 1:length(coordlist)
                obj.handlelist.coordinates.(coordlist{i}) = [];
            end
            
            paramlist = system.getParameterList();
            obj.handlelist.parameters = struct();
            for i = 1:length(paramlist)
                obj.handlelist.parameters.(paramlist{i}) = [];
            end

            obj.handlelist.multipliers = struct();
            for i = 1:length(coordlist)
                obj.handlelist.multipliers.(['mod_' num2str(i)]) = [];
            end
            for i = 1:length(coordlist)
                obj.handlelist.multipliers.(['arg_' num2str(i)]) = [];
            end           
            
            
            testlist = curvetype.getTestfunctions();
            obj.handlelist.testfunctions = struct();
            for i = 1:length(testlist)
                obj.handlelist.testfunctions.( testlist{i} ) = [] ;
            end
            
            %TODO:
            obj.handlelist.user_functions = struct(); %
            %
            if (~isempty(obj.startdata))
                ufnr = obj.startdata.getNrUserfunctions();
                for i = 1:ufnr
                      if (  obj.startdata.isValidUserfunction(i) )
                          obj.handlelist.user_functions.(obj.startdata.getLabelUserfunction(i)) = [];
                      end 
                end
            end
            
            obj.handlelist.npoints = [];
            obj.handlelist.current_stepsize = [];
            
        end
        
        
        
        %%%%          %%%%            %         %%%%        %%%%        %%%%       %%%%%%%
        
        
        

        function setNumVarEnabled(obj , keyword , bool)
            
            if (obj.varlist.(keyword) ~= bool)
                obj.varlist.(keyword) = bool;
                obj.notify('structureChanged');
            end 
        end
        function bool = isNumVarEnabled(obj, keyword)
            bool = (isfield(obj.varlist, keyword) &&  obj.varlist.(keyword));
        end
        
        
        function bool = isNumVarAllowed(obj , keyword)
            bool =   (isfield(obj.varlist, keyword) &&  obj.varlist.(keyword));
            if (bool &&  ismember(keyword , {'testfunctions' , 'user_functions' , 'multipliers' } ))
                subkeys = fields(obj.handlelist.(keyword));
                
                for i = 1:length(subkeys)
                    if (isFieldEnabled(obj, keyword , subkeys{i}))
                        return
                    end
                end
                bool = false;
                return;
            else
               return; 
            end
        end   
        
        

        function list = getNumVarList(obj , keyword)
            list = fieldnames(obj.handlelist.(keyword));
        end
        function  setNumHandle(obj , key, subkey, handle)
            obj.handlelist.(key).(subkey) = handle;
        end
        function  val = getNumHandle(obj , key, subkey)
            val = obj.handlelist.(key).(subkey);
        end
        %%%%
        function val = getNumHandle_s(obj,key)
            val = obj.handlelist.(key);
        end
        function setNumHandle_s(obj,key,val)
            obj.handlelist.(key) = val;
        end
        %%%%%
        

        
        function setValue(obj, key, subkey , value)
            set( obj.handlelist.(key).(subkey) , 'String' , num2str(value ,'%0.8g') );
        end
        function setValue_s(obj,key, value)
            set(obj.handlelist.(key), 'String' , num2str(value ,'%0.8g') );
        end
        
        function bool = containsSubVars(obj,key)
            bool  = isstruct( obj.handlelist.(key));
        end
        
        
        function list = getAllowed(obj)
            list = {};
            fieldlist = fieldnames(obj.varlist);
            for i = 1:length(fieldlist)
                if ( obj.isNumVarAllowed(fieldlist{i}))
                    list{end+1} = fieldlist{i};
                end
            end
        end
        
        function list = getOptions(obj)
            list = fieldnames(obj.varlist);
        end
        
        %%%%%%%%%%%%%%%%%%%5
        
        function b = isFieldEnabled(obj , key, subkey)
            
            
            b = true;
            
            if (isempty(obj.startdata)), return; end
            
            switch (key)
                case 'testfunctions'
                    b = obj.startdata.getMonitorSingularities(subkey);
                    return
                    
                case 'user_functions'
                    b = obj.startdata.isEnabledUserfunction(obj.startdata.getUserFunctionIndex(subkey));
                    return
                case 'multipliers'
                    b = obj.startdata.getMultipliers();
                    return

                otherwise
                    return
                    
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%
        
    function copyOver(obj,numdata)
        obj.varlist = numdata.varlist;
        printc(mfilename, 'copyOver');
        obj.notify('structureChanged');
        obj.notify('settingChanged');
    end
    
    function A = saveobj(obj)
        A.varlist = obj.varlist;
    end
    
    end
end

